
define([
        'jquery',
        'matchMedia',
        'jquery/ui',
        'mage/menu' ],
    function($, mediaCheck){
        $.widget('filoblu.evaMenu', $.mage.menu,  {
            options: {
                responsive: true,
                expanded: false,
                showDelay: 0,
                hideDelay: 0,
                mediaBreakpoint: '(max-width: 991px)',
                animation: {effect:'fade'},
                animationDelay: false
            },

            _toggleDesktopMode: function () {
                var categoryParent, html;

                this._on({
                    /**
                     * Prevent focus from sticking to links inside menu after clicking
                     * them (focus should always stay on UL during navigation).
                     */
                    'mousedown .ui-menu-item > a': function (event) {
                        event.preventDefault();
                    },

                    /**
                     * Prevent focus from sticking to links inside menu after clicking
                     * them (focus should always stay on UL during navigation).
                     */
                    'click .ui-state-disabled > a': function (event) {
                        event.preventDefault();
                    },

                    /**
                     * @param {jQuer.Event} event
                     */
                    'click .ui-menu-item:has(a)': function (event) {
                        var target = $(event.target).closest('.ui-menu-item');

                        if (!this.mouseHandled && target.not('.ui-state-disabled').length) {
                            this.select(event);

                            // Only set the mouseHandled flag if the event will bubble, see #9469.
                            if (!event.isPropagationStopped()) {
                                this.mouseHandled = true;
                            }

                            // Open submenu on click
                            if (target.has('.ui-menu').length) {
                                this.expand(event);
                            } else if (!this.element.is(':focus') &&
                                $(this.document[0].activeElement).closest('.ui-menu').length
                            ) {
                                // Redirect focus to the menu
                                this.element.trigger('focus', [true]);

                                // If the active item is on the top level, let it stay active.
                                // Otherwise, blur the active item since it is no longer visible.
                                if (this.active && this.active.parents('.ui-menu').length === 1) { //eslint-disable-line
                                    clearTimeout(this.timer);
                                }
                            }
                        }
                    },

                    /**
                     * @param {jQuery.Event} event
                     */
                    'mouseenter .ui-menu-item': function (event) {
                        var target = $(event.currentTarget);

                        // Remove ui-state-active class from siblings of the newly focused menu item
                        // to avoid a jump caused by adjacent elements both having a class with a border
                        target.siblings().children('.ui-state-active').removeClass('ui-state-active');
                        this.focus(event, target);
                    },

                    /**
                     * @param {jQuery.Event} event
                     */
                    'mouseleave': function (event) {
                        this.collapseAll(event, true);
                    },

                    /**
                     * Mouse leave.
                     */
                    'mouseleave .ui-menu': 'collapseAll',
                });

                categoryParent = this.element.find('.all-category');
                html = $('html');

                categoryParent.remove();

                if (html.hasClass('nav-open')) {
                    html.removeClass('nav-open');
                    setTimeout(function () {
                        html.removeClass('nav-before-open');
                    }, 300);
                }
            },

            /**
             * @private
             */
            _toggleMobileMode: function () {

                $(this.element).off('mouseenter mouseleave');
                this._on({
                    /**
                     * @param {jQuery.Event} event
                     */
                    'click .ui-menu-item:has(a)': function (event) {
                        var target;

                        event.preventDefault();
                        target = $(event.target).closest('.ui-menu-item');

                        if (!target.hasClass('with-children') || !target.has('.ui-menu').length) {
                            window.location.href = target.find('> a').attr('href');
                        }
                    },

                    /**
                     * @param {jQuery.Event} event
                     */
                    'click .ui-menu-item:has(.ui-state-active)': function (event) {
                        this.collapseAll(event, false);
                    }
                });

            },

            /**
             * @param {*} submenu
             * @private
             */
            _open: function (submenu) {
                var position = $.extend({
                    of: this.active
                }, this.options.position);

                var animation = this.options.animation.effect,
                    animationDirection = this.options.animation.direction,
                    animationDelay = this.options.animationDelay;

                mediaCheck({
                    media: this.options.mediaBreakpoint,
                    entry: $.proxy(function () {
                        this.element.find('.ui-menu').not(submenu.parents('.ui-menu'))
                            .slideUp()
                            .attr('aria-hidden', 'true');
                    }, this),
                    exit: $.proxy(function () {
                        clearTimeout(this.timer);
                        this.element.find('.ui-menu').not(submenu.parents('.ui-menu'))
                            .hide(animation, {direction: animationDirection}, animationDelay)
                            .attr('aria-hidden', 'true');
                    }, this)
                });

                this._openSubmenu(submenu, position)
            },

            /**
             * @param {*} submenu
             * @param {*} position
             * @private
             */
            _openSubmenu: function (submenu, position) {
                var animation = this.options.animation.effect,
                    animationDirection = this.options.animation.direction,
                    animationDelay = this.options.animationDelay;

                mediaCheck({
                    media: this.options.mediaBreakpoint,
                    entry: $.proxy(function () {
                        submenu
                            .slideDown()
                            .removeAttr('aria-hidden')
                            .attr('aria-expanded', 'true')
                            .position(position);
                    }, this),
                    exit: $.proxy(function () {
                        submenu
                            .show(animation, {direction: animationDirection}, animationDelay)
                            .removeAttr('aria-hidden')
                            .attr('aria-expanded', 'true')
                            .position(position);
                    }, this)
                });
            },

            // With no arguments, closes the currently active menu - if nothing is active
            // it closes all menus.  If passed an argument, it will search for menus BELOW
            /**
             * With no arguments, closes the currently active menu - if nothing is active
             * it closes all menus.  If passed an argument, it will search for menus BELOW.
             *
             * @param {*} startMenu
             * @private
             */
            _close: function (startMenu) {


                var animation = this.options.animation.effect,
                    animationDirection = this.options.animation.direction,
                    animationDelay = this.options.animationDelay;

                if (!startMenu) {
                    startMenu = this.active ? this.active.parent() : this.element;
                }

                mediaCheck({
                    media: this.options.mediaBreakpoint,
                    entry: $.proxy(function () {
                        startMenu
                            .find('.ui-menu')
                            .slideUp()
                            .attr('aria-hidden', 'true')
                            .attr('aria-expanded', 'false')
                            .end()
                            .find('a.ui-state-active')
                            .removeClass('ui-state-active');
                    }, this),
                    exit: $.proxy(function () {
                        startMenu
                            .find('.ui-menu')
                            .hide(animation, {direction: animationDirection}, animationDelay)
                            .attr('aria-hidden', 'true')
                            .attr('aria-expanded', 'false')
                            .end()
                            .find('a.ui-state-active')
                            .removeClass('ui-state-active');
                    }, this)
                });
            }

        });
        return $.filoblu.evaMenu;
    });